// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

#include "server/grpc_impl/request/CmdRequest.h"
#include "scheduler/SchedInst.h"

#include <memory>

namespace milvus {
namespace server {
namespace grpc {

CmdRequest::CmdRequest(const std::string& cmd, std::string& result)
    : GrpcBaseRequest(INFO_REQUEST_GROUP), cmd_(cmd), result_(result) {
}

BaseRequestPtr
CmdRequest::Create(const std::string& cmd, std::string& result) {
    return std::shared_ptr<GrpcBaseRequest>(new CmdRequest(cmd, result));
}

Status
CmdRequest::OnExecute() {
    if (cmd_ == "version") {
        result_ = MILVUS_VERSION;
    } else if (cmd_ == "tasktable") {
        result_ = scheduler::ResMgrInst::GetInstance()->DumpTaskTables();
    } else {
        result_ = "OK";
    }

    return Status::OK();
}

}  // namespace grpc
}  // namespace server
}  // namespace milvus
